package tk.antoine.roux.infrastructure;

import org.springframework.http.HttpStatus;
import org.springframework.http.ProblemDetail;
import org.springframework.http.ResponseEntity;
import org.springframework.web.ErrorResponse;
import tk.antoine.roux.domain.model.Either;

public interface Manager {

    static <E extends Exception, T> ResponseEntity<T> eitherToResponseEntity(Either<E, T> either, String errorMessage) {
        ResponseEntity<T> response;
        if (either.isLeft() ) {

            final ProblemDetail problemDetail;
            if (either.left() instanceof ErrorResponse exception) {
                problemDetail = exception.getBody();
            } else {
                problemDetail = ProblemDetail.forStatusAndDetail(HttpStatus.INTERNAL_SERVER_ERROR, either.left().getMessage());
            }

            problemDetail.setTitle(errorMessage);
            response = ResponseEntity.of(problemDetail).build();
        } else {
            response = ResponseEntity.ok(either.right());
        }
        return response;
    }

}
